    ".br.com",	"whois.centralnic.net",
    ".cn.com",	"whois.centralnic.net",
    ".de.com",	"whois.centralnic.net",
    ".eu.com",	"whois.centralnic.net",
    ".gb.com",	"whois.centralnic.net",
    ".gb.net",	"whois.centralnic.net",
    ".hu.com",	"whois.centralnic.net",
    ".no.com",	"whois.centralnic.net",
    ".qc.com",	"whois.centralnic.net",
    ".ru.com",	"whois.centralnic.net",
    ".sa.com",	"whois.centralnic.net",
    ".se.com",	"whois.centralnic.net",
    ".se.net",	"whois.centralnic.net",
    ".uk.com",	"whois.centralnic.net",
    ".uk.net",	"whois.centralnic.net",
    ".us.com",	"whois.centralnic.net",
    ".uy.com",	"whois.centralnic.net",
    ".za.com",	"whois.centralnic.net",
    ".jpn.com",	"whois.centralnic.net",
    ".web.com",	"whois.centralnic.net",
    ".com",	"\x04whois.crsnic.net",
    ".za.net",	"whois.za.net",
    ".net",	"\x04whois.crsnic.net",
    ".eu.org",	"whois.eu.org",
    ".za.org",	"whois.za.org",
    ".org",	"\x07",
    ".edu",	"whois.educause.net",
    ".gov",	"whois.nic.gov",
    ".int",	"whois.iana.org",
    ".mil",	"\x03",
    ".e164.arpa",	"whois.ripe.net",
    ".in-addr.arpa",	"\x0C",
    ".arpa",	"whois.iana.org",
    ".aero",	"whois.aero",
    ".asia",	"whois.nic.asia",
    ".biz",	"whois.nic.biz",
    ".cat",	"whois.cat",
    ".coop",	"whois.nic.coop",
    ".info",	"whois.afilias.info",
    ".jobs",	"\x04jobswhois.verisign-grs.com",
    ".mobi",	"whois.dotmobiregistry.net",
    ".museum",	"whois.museum",
    ".name",	"whois.nic.name",
    ".pro",	"whois.registrypro.pro",
    ".tel",	"whois.nic.tel",
    ".travel",	"whois.nic.travel",
    ".ac",	"whois.nic.ac",
    ".ad",	"\x03",
    ".ae",	"whois.aeda.net.ae",
    ".af",	"whois.nic.af",
    ".ag",	"whois.nic.ag",
    ".ai",	"whois.ai",
    ".al",	"\x03",
    ".am",	"whois.nic.am",
    ".an",	"\x03",
    ".ao",	"\x03",
    ".aq",	"\x03",
    ".ar",	"\x01http://www.nic.ar/",
    ".as",	"whois.nic.as",
    ".priv.at",	"whois.nic.priv.at",
    ".at",	"whois.nic.at",
    ".au",	"whois.ausregistry.net.au",
    ".aw",	"\x03",
    ".ax",	"\x03",
    ".az",	"\x01http://www.nic.az/AzCheck.htm",
    ".ba",	"\x01http://www.nic.ba/stream/whois/",
    ".bb",	"\x01http://www.barbadosdomains.net/search_domain.php",
    ".bd",	"\x01http://www.whois.com.bd/",
    ".be",	"whois.dns.be",
    ".bf",	"\x03",
    ".bg",	"whois.register.bg",
    ".bh",	"\x03",
    ".bi",	"\x01http://whois.nic.bi/register/whois.hei",
    ".bj",	"whois.nic.bj",
    ".bm",	"\x01http://207.228.133.14/cgi-bin/lansaweb?procfun+BMWHO+BMWHO2+WHO",
    ".bn",	"\x03",
    ".bo",	"\x01http://www.nic.bo/",
    ".br",	"whois.nic.br",
    ".bs",	"\x01http://www.nic.bs/cgi-bin/search.pl",
    ".bt",	"\x01http://www.nic.bt/",
    ".bv",	"\x03",
    ".by",	"\x01http://www.tld.by/indexeng.html",
    ".bw",	"\x03",
    ".bz",	"\x08",
    ".co.ca",	"whois.co.ca",
    ".ca",	"whois.cira.ca",
    ".cc",	"\x04whois.nic.cc",
    ".cd",	"whois.nic.cd",
    ".cf",	"\x03",
    ".cg",	"\x01http://www.nic.cg/cgi-bin/whois.pl",
    ".ch",	"whois.nic.ch",
    ".ci",	"www.nic.ci",
    ".ck",	"whois.nic.ck",
    ".cl",	"whois.nic.cl",
    ".cm",	"\x01http://netcom.cm/whois.php",
    ".edu.cn",	"whois.edu.cn",
    ".cn",	"whois.cnnic.cn",
    ".uk.co",	"whois.uk.co",
    ".co",	"\x01https://www.nic.co/",
    ".cr",	"\x01http://www.nic.cr/niccr_publico/showRegistroDominiosScreen.do",
    ".cu",	"\x01http://www.nic.cu/consult.html",
    ".cv",	"\x03",
    ".cx",	"whois.nic.cx",
    ".cy",	"\x01http://www.nic.cy/nslookup/online_database.php",
    ".cz",	"whois.nic.cz",
    ".de",	"whois.denic.de",
    ".dj",	"\x01http://www.nic.dj/whois.php",
    ".dk",	"whois.dk-hostmaster.dk",
    ".dm",	"whois.nic.dm",
    ".do",	"\x01http://www.nic.do/whois-h.php3",
    ".dz",	"\x01https://www.nic.dz/",
    ".ec",	"\x01http://www.nic.ec/whois/eng/whois.asp",
    ".ee",	"whois.eenet.ee",
    ".eg",	"\x03",
    ".er",	"\x03",
    ".es",	"\x01https://www.nic.es/",
    ".et",	"\x03",
    ".eu",	"whois.eu",
    ".fi",	"whois.ficora.fi",
    ".fj",	"whois.usp.ac.fj",
    ".fk",	"\x03",
    ".fm",	"\x01http://www.dot.fm/whois.html",
    ".fo",	"whois.ripe.net",
    ".fr",	"whois.nic.fr",
    ".ga",	"\x03",
    ".gb",	"\x03",
    ".gd",	"whois.adamsnames.tc",
    ".ge",	"\x01http://www.registration.ge/",
    ".gf",	"\x01http://www.nic.gf/?id=whois",
    ".gg",	"whois.gg",
    ".gh",	"\x01http://www.nic.gh/customer/search_c.htm",
    ".gi",	"\x08",
    ".gl",	"whois.nic.gl",
    ".gm",	"whois.ripe.net",
    ".gn",	"\x03",
    ".gp",	"whois.nic.gp",
    ".gq",	"\x03",
    ".gr",	"\x01https://grweb.ics.forth.gr/Whois?lang=en",
    ".gs",	"whois.nic.gs",
    ".gt",	"\x01http://www.gt/whois.html",
    ".gu",	"\x01http://gadao.gov.gu/domainsearch.htm",
    ".gw",	"\x03",
    ".gy",	"whois.registry.gy",
    ".hk",	"whois.hkdnr.net.hk",
    ".hm",	"whois.registry.hm",
    ".hn",	"\x08",
    ".hr",	"\x01http://www.dns.hr/pretrazivanje.html",
    ".ht",	"whois.nic.ht",
    ".hu",	"whois.nic.hu",
    ".id",	"whois.pandi.or.id",
    ".ie",	"whois.domainregistry.ie",
    ".il",	"whois.isoc.org.il",
    ".im",	"whois.nic.im",
    ".in",	"whois.registry.in",
    ".io",	"whois.nic.io",
    ".iq",	"\x03",
    ".ir",	"whois.nic.ir",
    ".is",	"whois.isnic.is",
    ".it",	"whois.nic.it",
    ".je",	"whois.je",
    ".jm",	"\x03",
    ".jo",	"\x01http://www.dns.jo/Whois.aspx",
    ".jp",	"whois.jprs.jp",
    ".ke",	"whois.kenic.or.ke",
    ".kg",	"whois.domain.kg",
    ".kh",	"\x03",
    ".ki",	"whois.nic.mu",
    ".km",	"\x03",
    ".kn",	"\x01http://www.nic.kn/",
    ".kp",	"whois.kcce.kp",
    ".kr",	"whois.nic.or.kr",
    ".kw",	"\x01http://www.kw/",
    ".ky",	"\x01http://kynseweb.messagesecure.com/kywebadmin/",
    ".kz",	"whois.nic.kz",
    ".la",	"whois.nic.la",
    ".lb",	"\x01http://www.aub.edu.lb/lbdr/search.html",
    ".lc",	"\x08",
    ".li",	"whois.nic.li",
    ".lk",	"whois.nic.lk",
    ".lr",	"\x03",
    ".ls",	"\x01http://www.co.ls/co.asp",
    ".lt",	"whois.domreg.lt",
    ".lu",	"whois.dns.lu",
    ".lv",	"whois.nic.lv",
    ".ly",	"whois.nic.ly",
    ".ma",	"whois.iam.net.ma",
    ".mc",	"whois.ripe.net",
    ".md",	"\x01http://www.dns.md/wh1.php",
    ".me",	"whois.meregistry.net",
    ".mg",	"whois.nic.mg",
    ".mh",	"\x03",
    ".mk",	"\x01http://dns.marnet.net.mk/registar.php",
    ".ml",	"\x03",
    ".mm",	"\x03",
    ".mn",	"\x08",
    ".mo",	"\x01http://www.monic.net.mo/",
    ".mp",	"\x03",
    ".mq",	"\x03",
    ".mr",	"\x03",
    ".ms",	"whois.nic.ms",
    ".mt",	"\x01https://www.nic.org.mt/dotmt/",
    ".mu",	"whois.nic.mu",
    ".mv",	"\x03",
    ".mw",	"\x01http://www.registrar.mw/",
    ".mx",	"whois.nic.mx",
    ".my",	"whois.domainregistry.my",
    ".mz",	"\x03",
    ".na",	"whois.na-nic.com.na",
    ".nc",	"whois.cctld.nc",
    ".ne",	"\x03",
    ".nf",	"whois.nic.nf",
    ".ng",	"whois.register.net.ng",
    ".ni",	"\x01http://www.nic.ni/consulta.htm",
    ".nl",	"whois.domain-registry.nl",
    ".no",	"whois.norid.no",
    ".np",	"\x01http://register.mos.com.np/userSearchInc.asp",
    ".nr",	"\x01http://www.cenpac.net.nr/dns/whois.html",
    ".nu",	"whois.nic.nu",
    ".nz",	"whois.srs.net.nz",
    ".om",	"\x01http://www.omnic.om/onlineUser/WHOISLookup.jsp",
    ".pa",	"\x01http://www.nic.pa/",
    ".pe",	"whois.nic.pe",
    ".pf",	"\x03",
    ".pg",	"\x03",
    ".ph",	"\x01http://www.dot.ph/",
    ".pk",	"\x01http://www.pknic.net.pk/",
    ".co.pl",	"whois.co.pl",
    ".pl",	"whois.dns.pl",
    ".pm",	"whois.nic.fr",
    ".pn",	"\x01http://www.pitcairn.pn/PnRegistry/",
    ".pr",	"whois.nic.pr",
    ".ps",	"\x01http://www.nic.ps/whois/whois.html",
    ".pt",	"whois.dns.pt",
    ".pw",	"\x03",
    ".py",	"\x01http://www.nic.py/consultas.html",
    ".qa",	"\x03",
    ".re",	"whois.nic.fr",
    ".ro",	"whois.rotld.ro",
    ".rs",	"\x01http://www.nic.rs/en/whois",
    ".edu.ru",	"whois.informika.ru",
    ".ru",	"whois.ripn.net",
    ".rw",	"\x01http://www.nic.rw/cgi-bin/whoisrw.pl",
    ".sa",	"whois.nic.net.sa",
    ".sb",	"whois.coccaregistry.net",
    ".sc",	"\x08",
    ".sd",	"\x03",
    ".se",	"whois.nic-se.se",
    ".sg",	"whois.nic.net.sg",
    ".sh",	"whois.nic.sh",
    ".si",	"whois.arnes.si",
    ".sj",	"\x03",
    ".sk",	"whois.sk-nic.sk",
    ".sl",	"whois.nic.sl",
    ".sm",	"whois.ripe.net",
    ".sn",	"whois.nic.sn",
    ".so",	"\x03",
    ".sr",	"\x03",
    ".st",	"whois.nic.st",
    ".su",	"whois.ripn.net",
    ".sv",	"\x01http://www.uca.edu.sv/dns/",
    ".sy",	"\x03",
    ".sz",	"\x03",
    ".tc",	"whois.adamsnames.tc",
    ".td",	"\x03",
    ".tf",	"whois.nic.tf",
    ".tg",	"\x01http://www.nic.tg/",
    ".th",	"whois.thnic.net",
    ".tj",	"\x01http://www.nic.tj/whois.html",
    ".tk",	"whois.dot.tk",
    ".tl",	"whois.nic.tl",
    ".tm",	"whois.nic.tm",
    ".tn",	"\x01http://whois.ati.tn/",
    ".to",	"whois.tonic.to",
    ".tp",	"\x03",
    ".tr",	"whois.nic.tr",
    ".tt",	"\x01http://www.nic.tt/cgi-bin/search.pl",
    ".tv",	"\x04whois.nic.tv",
    ".tw",	"whois.twnic.net",
    ".tz",	"\x01http://whois.tznic.or.tz/",
    ".ua",	"whois.net.ua",
    ".ug",	"www.registry.co.ug",
    ".ac.uk",	"whois.ja.net",
    ".bl.uk",	"\x03",
    ".british-library.uk",	"\x03",
    ".gov.uk",	"whois.ja.net",
    ".icnet.uk",	"\x03",
    ".jet.uk",	"\x03",
    ".mod.uk",	"\x03",
    ".nhs.uk",	"\x03",
    ".nls.uk",	"\x03",
    ".parliament.uk",	"\x03",
    ".police.uk",	"\x03",
    ".uk",	"whois.nic.uk",
    ".fed.us",	"whois.nic.gov",
    ".us",	"whois.nic.us",
    ".com.uy",	"\x01https://nic.anteldata.com.uy/dns/",
    ".uy",	"whois.nic.org.uy",
    ".uz",	"whois.cctld.uz",
    ".va",	"whois.ripe.net",
    ".vc",	"\x08",
    ".ve",	"whois.nic.ve",
    ".vg",	"whois.adamsnames.tc",
    ".vi",	"\x01http://www.nic.vi/whoisform.htm",
    ".vn",	"\x01http://www.vnnic.vn/english/",
    ".vu",	"\x01http://www.vunic.vu/whois.html",
    ".wf",	"whois.nic.fr",
    ".ws",	"whois.samoanic.ws",
    ".ye",	"\x03",
    ".yt",	"whois.nic.fr",
    ".ac.za",	"whois.ac.za",
    ".co.za",	"whois.coza.net.za",
    ".gov.za",	"whois.gov.za",
    ".org.za",	"\x01http://www.org.za/",
    ".za",	"\x03",
    ".zm",	"\x03",
    ".zw",	"\x03",
    ".xn--fiqs8s",	"cwhois.cnnic.cn",
    ".xn--fiqz9s",	"cwhois.cnnic.cn",
    ".xn--mgbaam7a8h",	"whois.aeda.net.ae",
    ".xn--mgberp4a5d4ar",	"whois.nic.net.sa",
    ".xn--wgbh1c",	"whois.dotmasr.eg",
    ".xn--p1ai",	"whois.ripn.net",
    "-dom",	"whois.networksolutions.com",
    "-org",	"whois.networksolutions.com",
    "-hst",	"whois.networksolutions.com",
    "-arin",	"whois.arin.net",
    "-ripe",	"whois.ripe.net",
    "-mnt",	"whois.ripe.net",
    "-lacnic",	"whois.lacnic.net",
    "-afrinic",	"whois.afrinic.net",
    "-ap",	"whois.apnic.net",
    "-ar",	"whois.aunic.net",
    "-cn",	"whois.cnnic.net.cn",
    "-cz",	"whois.nic.cz",
    "-dk",	"whois.dk-hostmaster.dk",
    "-il",	"whois.isoc.org.il",
    "-is",	"whois.isnic.is",
    "-kg",	"whois.domain.kg",
    "-ti",	"whois.telstra.net",
    "-tw",	"whois.twnic.net",
    "-6bone",	"whois.6bone.net",
    "-coop",	"whois.nic.coop",
    "-cknic",	"whois.nic.ck",
    "-idnic",	"whois.idnic.net.id",
    "-itnic",	"whois.nic.it",
    "-frnic",	"whois.nic.fr",
    "-gandi",	"whois.gandi.net",
    "-kenic",	"whois.kenic.or.ke",
    "-lrms",	"whois.afilias.info",
    "-metu",	"whois.metu.edu.tr",
    "-nicat",	"whois.nic.at",
    "-nicir",	"whois.nic.ir",
    "-norid",	"whois.norid.no",
    "-ripn",	"whois.ripn.net",
    "-rotld",	"whois.rotld.ro",
    "-sgnic",	"whois.nic.net.sg",
    "-tel",	"whois.nic.tel",
    "-uanic",	"whois.com.ua",
    "-uynic",	"www.rau.edu.uy",
    "-sixxs",	"whois.sixxs.net",
